@@ -14,6 +14,10 @@
 #   define _XOPEN_SOURCE 500
 #endif
 
+#ifdef _MSC_VER
+#define _USE_MATH_DEFINES
+#endif
+
 #include <ctype.h>
 #include <float.h>
 #include <limits.h>
@@ -56,7 +60,7 @@ asciiPrintProduct(
     const int* const		powers,
     const int			count,
     char* const			buf,
-    const size_t		max,
+    size_t		        max,
     IdGetter			getId);
 static int
 latin1PrintProduct(
@@ -64,7 +68,7 @@ latin1PrintProduct(
     const int* const		powers,
     const int			count,
     char* const			buf,
-    const size_t		max,
+    size_t		        max,
     IdGetter			getId);
 static int
 utf8PrintProduct(
@@ -72,7 +76,7 @@ utf8PrintProduct(
     const int* const		powers,
     const int			count,
     char* const			buf,
-    const size_t		max,
+    size_t		        max,
     IdGetter			getId);
 
 static ut_visitor	formatter;
@@ -779,7 +783,7 @@ latin1PrintProduct(
  *
  * Arguments:
  *	unit		Pointer to the product-unit to be formatted.
- *	count		The number of basic-units that constitute the 
+ *	count		The number of basic-units that constitute the
  *			product-unit.
  *	basicUnits	Pointer to pointers to the basic-units that constitute
  *			the product-unit.
@@ -819,7 +823,7 @@ formatProduct(
 	else {
             const char*	id = formatPar->getId(unit, formatPar->encoding);
 
-            nchar = 
+            nchar =
                 id == NULL
                     ? formatPar->printProduct(basicUnits, powers, count,
                         formatPar->buf, formatPar->size, formatPar->getId)
@@ -850,7 +854,7 @@ formatProduct(
  *	getDefinition	Returns the definition of "unit" in terms of basic
  *			units.
  *	encoding	The type of encoding to use.
- *	addParens	Whether or not to add bracketing parentheses if 
+ *	addParens	Whether or not to add bracketing parentheses if
  *			whitespace is printed.
  * Returns:
  *	-1		Failure.  See errno.
@@ -966,7 +970,7 @@ formatGalilean(
     else {
 	const char*	id = formatPar->getId(unit, formatPar->encoding);
 
-	nchar = 
+	nchar =
 	    id == NULL
 		? printGalilean(scale, underlyingUnit, offset, formatPar->buf,
 		    formatPar->size, formatPar->getId, formatPar->getDefinition,
@@ -1003,7 +1007,7 @@ formatGalilean(
  *	getDefinition	Returns the definition of "unit" in terms of basic
  *			units.
  *	encoding	The type of encoding to use.
- *	addParens	Whether or not to add bracketing parentheses if 
+ *	addParens	Whether or not to add bracketing parentheses if
  *			whitespace is printed.
  * Returns:
  *	-1		Failure.  See errno.
@@ -1050,7 +1054,9 @@ printTimestamp(
 	nchar = n < 0 ? n : nchar + n;
 
 	if (nchar >= 0) {
-	    int	useSeparators = useNames || year < 1000 || year > 9999;
+            size = SUBTRACT_SIZET(size, n);
+
+            int	useSeparators = useNames || year < 1000 || year > 9999;
 
 	    n =  snprintf(buf+nchar, size,
 		useSeparators
@@ -1067,7 +1073,9 @@ printTimestamp(
             }
 
 	    if (nchar >= 0) {
-		int	decimalCount = -(int)floor(log10(resolution));
+		int	decimalCount = resolution <= 0.0
+		        ? 9 // Nanosecond resolution
+		        : -(int)floor(log10(resolution));
 
 		if (decimalCount > -2) {
 		    n = snprintf(buf+nchar, size,
@@ -1083,7 +1091,7 @@ printTimestamp(
 		}			/* sufficient precision for seconds */
 
 		if (nchar >= 0) {
-                    n = snprintf(buf+nchar, size, "%s", 
+                    n = snprintf(buf+nchar, size, "%s",
                             addParens ? " UTC)" : " UTC");
                     if (0 > n) {
                         nchar = -1;
@@ -1144,7 +1152,7 @@ formatTimestamp(
     else {
 	const char*	id = formatPar->getId(unit, formatPar->encoding);
 
-	nchar = 
+	nchar =
 	    id == NULL
 		? printTimestamp(underlyingUnit, year, month, day, hour, minute,
 		    second, resolution, formatPar->buf, formatPar->size,
@@ -1176,7 +1184,7 @@ formatTimestamp(
  *	getDefinition	Returns the definition of "unit" in terms of basic
  *			units.
  *	encoding	The type of encoding to use.
- *	addParens	Whether or not to add bracketing parentheses if 
+ *	addParens	Whether or not to add bracketing parentheses if
  *			whitespace is printed.
  * Returns:
  *	-1		Failure.  See errno.
@@ -1255,7 +1263,7 @@ formatLogarithmic(
     else {
 	const char*	id = formatPar->getId(unit, formatPar->encoding);
 
-	nchar = 
+	nchar =
 	    id == NULL
 		? printLogarithmic(base, reference, formatPar->buf,
 		    formatPar->size, formatPar->getId, formatPar->getDefinition,
