@@ -11,6 +11,60 @@
 #include <stdarg.h>
 #include <stddef.h>
 
+#ifdef _MSC_VER
+#include <stdlib.h>
+#include <stdio.h>
+#include <stdarg.h>
+#include <io.h>
+
+#define _USE_MATH_DEFINES
+
+/* Define a bunch of variables to use the ISO C++ conformant name instead
+   of the POSIX name. This quiets a lot of the warnings thrown by MSVC. */
+#define read _read
+#define open _open
+#define close _close
+#define strdup _strdup
+#define strcasecmp stricmp
+#define stricmp _stricmp
+#define isatty _isatty
+
+//We must accommodate the lack of snprintf in versions of MSVC less than 1900.
+//udunits_snprintf is defined in udunits_snprintf.c, in lib/.
+#define snprintf udunits_snprintf
+
+int udunits_snprintf(
+   char* str,
+     size_t size,
+     const char* format,
+     ...);
+
+int udunits_vsnprintf(
+char* str,
+  size_t size,
+  const char* format,
+  va_list ap);
+
+#endif
+
+/* If we are working in Visual Studio and have a
+   shared library, we will need to do some slight-of-hand
+   in order to make it generate a proper export library.
+*/
+
+#if defined(DLL_UDUNITS2) /* define when library is a DLL */
+#   if defined(DLL_EXPORT) /* defined when building the library */
+#     define MSC_EXTRA __declspec(dllexport)
+#   else
+#     define MSC_EXTRA __declspec(dllimport)
+#   endif
+#else
+# define MSC_EXTRA
+#endif /* defined(DLL_UDUNITS2) */
+
+#define EXTERNL MSC_EXTRA extern
+
+
 #include "converter.h"
 
 typedef struct ut_system	ut_system;
@@ -63,7 +117,7 @@ typedef struct ut_visitor {
      *	UT_SUCCESS	Success.
      *	else		Failure.
      */
-    ut_status	(*visit_basic)(const ut_unit* unit, void* arg); 
+    ut_status	(*visit_basic)(const ut_unit* unit, void* arg);
 
     /*
      * Visits a product-unit.  A product-unit is a product of zero or more
@@ -81,7 +135,7 @@ typedef struct ut_visitor {
      *	else		Failure.
      */
     ut_status	(*visit_product)(const ut_unit* unit, int count,
-	const ut_unit* const* basicUnits, const int* powers, void* arg); 
+	const ut_unit* const* basicUnits, const int* powers, void* arg);
 
     /*
      * Visits a Galilean-unit.  A Galilean-unit has an underlying unit and a
@@ -99,7 +153,7 @@ typedef struct ut_visitor {
      *	else		Failure.
      */
     ut_status	(*visit_galilean)(const ut_unit* unit, double scale,
-	const ut_unit* underlyingUnit, double offset, void* arg); 
+	const ut_unit* underlyingUnit, double offset, void* arg);
 
     /*
      * Visits a timestamp-unit.  A timestamp-unit has an underlying unit of time
@@ -115,7 +169,7 @@ typedef struct ut_visitor {
      *	else		Failure.
      */
     ut_status	(*visit_timestamp)(const ut_unit* unit,
-	const ut_unit* timeUnit, double origin, void* arg); 
+	const ut_unit* timeUnit, double origin, void* arg);
 
     /*
      * Visits a logarithmic-unit.  A logarithmic-unit has a logarithmic base and
@@ -131,7 +185,7 @@ typedef struct ut_visitor {
      *	else		Failure.
      */
     ut_status	(*visit_logarithmic)(const ut_unit* unit, double base,
-	const ut_unit* reference, void* arg); 
+	const ut_unit* reference, void* arg);
 } ut_visitor;
 
 
@@ -139,7 +193,7 @@ typedef int (*ut_error_message_handler)(const char* fmt, va_list args);
 
 
 #ifdef __cplusplus
-extern "C" {
+EXTERNL "C" {
 #endif
 
 
@@ -159,7 +213,7 @@ extern "C" {
  *                  compile-time pathname of the installed, default, unit
  *                  database is returned.
  */
-const char*
+EXTERNL const char*
 ut_get_path_xml(
 	const char*	path,
 	ut_status*  status);
@@ -190,7 +244,7 @@ ut_get_path_xml(
  *		    UT_OS		Operating-system error.  See "errno".
  *	else	Pointer to the unit-system defined by "path".
  */
-ut_system*
+EXTERNL ut_system*
 ut_read_xml(
     const char*	path);
 
@@ -204,7 +258,7 @@ ut_read_xml(
  *		    UT_OS	Operating-system error.  See "errno".
  *	else	Pointer to a new unit system.
  */
-ut_system*
+EXTERNL ut_system*
 ut_new_system(void);
 
 
@@ -216,7 +270,7 @@ ut_new_system(void);
  *	system		Pointer to the unit-system to be freed.  Use of "system"
  *			upon return results in undefined behavior.
  */
-void
+EXTERNL void
 ut_free_system(
     ut_system*	system);
 
@@ -231,7 +285,7 @@ ut_free_system(
  *		    UT_BAD_ARG	"unit" is NULL.
  *	else	Pointer to the unit-system to which "unit" belongs.
  */
-ut_system*
+EXTERNL ut_system*
 ut_get_system(
     const ut_unit* const	unit);
 
@@ -249,7 +303,7 @@ ut_get_system(
  *		While not necessary, the pointer may be passed to ut_free()
  *		when the unit is no longer needed by the client.
  */
-ut_unit*
+EXTERNL ut_unit*
 ut_get_dimensionless_unit_one(
     const ut_system* const	system);
 
@@ -270,14 +324,14 @@ ut_get_dimensionless_unit_one(
  *		The pointer should be passed to ut_free() when the unit is
  *		no longer needed.
  */
-ut_unit*
+EXTERNL ut_unit*
 ut_get_unit_by_name(
     const ut_system* const	system,
     const char* const		name);
 
 
 /*
- * Returns the unit with a given symbol from a unit-system.  Symbol 
+ * Returns the unit with a given symbol from a unit-system.  Symbol
  * comparisons are case-sensitive.
  *
  * Arguments:
@@ -293,7 +347,7 @@ ut_get_unit_by_name(
  *		The pointer should be passed to ut_free() when the unit is no
  *		longer needed.
  */
-ut_unit*
+EXTERNL ut_unit*
 ut_get_unit_by_symbol(
     const ut_system* const	system,
     const char* const		symbol);
@@ -312,7 +366,7 @@ ut_get_unit_by_symbol(
  *			belongs is set to a different unit.
  *	UT_SUCCESS	Success.
  */
-ut_status
+EXTERNL ut_status
 ut_set_second(
     const ut_unit* const	second);
 
@@ -337,7 +391,7 @@ ut_set_second(
  *	UT_EXISTS	"name" already maps to a different value.
  *	UT_OS		Operating-system failure.  See "errno".
  */
-ut_status
+EXTERNL ut_status
 ut_add_name_prefix(
     ut_system* const	system,
     const char* const	name,
@@ -360,7 +414,7 @@ ut_add_name_prefix(
  *	UT_EXISTS	"symbol" already maps to a different value.
  *	UT_OS		Operating-system failure.  See "errno".
  */
-ut_status
+EXTERNL ut_status
 ut_add_symbol_prefix(
     ut_system* const	system,
     const char* const	symbol,
@@ -386,7 +440,7 @@ ut_add_symbol_prefix(
  *		ut_free() when the unit is no longer needed by the client (the
  *		unit will remain in the unit-system).
  */
-ut_unit*
+EXTERNL ut_unit*
 ut_new_base_unit(
     ut_system* const	system);
 
@@ -407,7 +461,7 @@ ut_new_base_unit(
  *		passed to ut_free() when the unit is no longer needed by the
  *		client (the unit will remain in the unit-system).
  */
-ut_unit*
+EXTERNL ut_unit*
 ut_new_dimensionless_unit(
     ut_system* const	system);
 
@@ -425,9 +479,9 @@ ut_new_dimensionless_unit(
  *		passed to ut_free() when the unit is no longer needed by the
  *		client.
  */
-ut_unit*
+EXTERNL ut_unit*
 ut_clone(
-    const ut_unit*	unit);
+    const ut_unit* const unit);
 
 
 /*
@@ -438,7 +492,7 @@ ut_clone(
  * Arguments:
  *	unit	Pointer to the unit to have its resources freed or NULL.
  */
-void
+EXTERNL void
 ut_free(
     ut_unit* const	unit);
 
@@ -462,7 +516,7 @@ ut_free(
  *	else		Pointer to the name in the given encoding to which
  *			"unit" maps.
  */
-const char*
+EXTERNL const char*
 ut_get_name(
     const ut_unit* const	unit,
     const ut_encoding		encoding);
@@ -483,7 +537,7 @@ ut_get_name(
  *	UT_EXISTS	"name" already maps to a different unit.
  *	UT_SUCCESS	Success.
  */
-ut_status
+EXTERNL ut_status
 ut_map_name_to_unit(
     const char* const		name,
     const ut_encoding		encoding,
@@ -502,7 +556,7 @@ ut_map_name_to_unit(
  *	UT_SUCCESS	Success.
  *	UT_BAD_ARG	"system" or "name" is NULL.
  */
-ut_status
+EXTERNL ut_status
 ut_unmap_name_to_unit(
     ut_system*		system,
     const char* const	name,
@@ -525,7 +579,7 @@ ut_unmap_name_to_unit(
  *	UT_OS		Operating-system error.  See "errno".
  *	UT_EXISTS	"unit" already maps to a name.
  */
-ut_status
+EXTERNL ut_status
 ut_map_unit_to_name(
     const ut_unit* const	unit,
     const char* const		name,
@@ -543,7 +597,7 @@ ut_map_unit_to_name(
  *	UT_BAD_ARG	"unit" is NULL.
  *	UT_SUCCESS	Success.
  */
-ut_status
+EXTERNL ut_status
 ut_unmap_unit_to_name(
     const ut_unit* const	unit,
     ut_encoding			encoding);
@@ -568,7 +622,7 @@ ut_unmap_unit_to_name(
  *	else		Pointer to the symbol in the given encoding to which
  *			"unit" maps.
  */
-const char*
+EXTERNL const char*
 ut_get_symbol(
     const ut_unit* const	unit,
     const ut_encoding	encoding);
@@ -589,7 +643,7 @@ ut_get_symbol(
  *	UT_EXISTS	"symbol" already maps to a different unit.
  *	UT_SUCCESS	Success.
  */
-ut_status
+EXTERNL ut_status
 ut_map_symbol_to_unit(
     const char* const		symbol,
     const ut_encoding		encoding,
@@ -608,7 +662,7 @@ ut_map_symbol_to_unit(
  *	UT_SUCCESS	Success.
  *	UT_BAD_ARG	"system" or "symbol" is NULL.
  */
-ut_status
+EXTERNL ut_status
 ut_unmap_symbol_to_unit(
     ut_system*		system,
     const char* const	symbol,
@@ -630,7 +684,7 @@ ut_unmap_symbol_to_unit(
  *	UT_OS		Operating-system error.  See "errno".
  *	UT_EXISTS	"unit" already maps to a symbol.
  */
-ut_status
+EXTERNL ut_status
 ut_map_unit_to_symbol(
     const ut_unit*		unit,
     const char* const		symbol,
@@ -649,7 +703,7 @@ ut_map_unit_to_symbol(
  *	UT_SUCCESS	Success.
  *	UT_BAD_ARG	"unit" is NULL.
  */
-ut_status
+EXTERNL ut_status
 ut_unmap_unit_to_symbol(
     const ut_unit* const	unit,
     ut_encoding			encoding);
@@ -673,7 +727,7 @@ ut_unmap_unit_to_symbol(
  *		    UT_SUCCESS		"unit" is dimensionfull.
  *	else	"unit" is dimensionless.
  */
-int
+EXTERNL int
 ut_is_dimensionless(
     const ut_unit* const	unit);
 
@@ -692,7 +746,7 @@ ut_is_dimensionless(
  *						unit-systems.
  *	else		The units belong to the same unit-system.
  */
-int
+EXTERNL int
 ut_same_system(
     const ut_unit* const	unit1,
     const ut_unit* const	unit2);
@@ -712,7 +766,7 @@ ut_same_system(
  *	 0	The first and second units are equal or both units are NULL.
  *	>0	The first unit is greater than the second unit.
  */
-int
+EXTERNL int
 ut_compare(
     const ut_unit* const	unit1,
     const ut_unit* const	unit2);
@@ -720,7 +774,7 @@ ut_compare(
 
 /*
  * Indicates if numeric values in one unit are convertible to numeric values in
- * another unit via "ut_get_converter()".  In making this determination, 
+ * another unit via "ut_get_converter()".  In making this determination,
  * dimensionless units are ignored.
  *
  * Arguments:
@@ -737,7 +791,7 @@ ut_compare(
  *						"kilogram").
  *	else	Numeric values can be converted between the units.
  */
-int
+EXTERNL int
 ut_are_convertible(
     const ut_unit* const	unit1,
     const ut_unit* const	unit2);
@@ -766,7 +820,7 @@ ut_are_convertible(
  *			should be passed to cv_free() when no longer needed by
  *			the client.
  */
-cv_converter*
+EXTERNL cv_converter*
 ut_get_converter(
     ut_unit* const	from,
     ut_unit* const	to);
@@ -795,7 +849,7 @@ ut_get_converter(
  *			passed to ut_free() when the unit is no longer needed by
  *			the client.
  */
-ut_unit*
+EXTERNL ut_unit*
 ut_scale(
     const double		factor,
     const ut_unit* const	unit);
@@ -819,7 +873,7 @@ ut_scale(
  *			passed to ut_free() when the unit is no longer needed by
  *			the client.
  */
-ut_unit*
+EXTERNL ut_unit*
 ut_offset(
     const ut_unit* const	unit,
     const double	offset);
@@ -847,7 +901,7 @@ ut_offset(
  *	else	Pointer to the resulting unit.  The pointer should be passed
  *		to ut_free() when the unit is no longer needed by the client.
  */
-ut_unit*
+EXTERNL ut_unit*
 ut_offset_by_time(
     const ut_unit* const	unit,
     const double	origin);
@@ -868,7 +922,7 @@ ut_offset_by_time(
  *	else	Pointer to the resulting unit.  The pointer should be passed
  *		to ut_free() when the unit is no longer needed by the client.
  */
-ut_unit*
+EXTERNL ut_unit*
 ut_multiply(
     const ut_unit* const	unit1,
     const ut_unit* const	unit2);
@@ -887,7 +941,7 @@ ut_multiply(
  *	else	Pointer to the resulting unit.  The pointer should be passed to
  *		ut_free() when the unit is no longer needed by the client.
  */
-ut_unit*
+EXTERNL ut_unit*
 ut_invert(
     const ut_unit* const	unit);
 
@@ -913,7 +967,7 @@ ut_invert(
  *	else	Pointer to the resulting unit.  The pointer should be passed to
  *		ut_free() when the unit is no longer needed by the client.
  */
-ut_unit*
+EXTERNL ut_unit*
 ut_divide(
     const ut_unit* const	numer,
     const ut_unit* const	denom);
@@ -924,7 +978,7 @@ ut_divide(
  *
  * Arguments:
  *	unit	Pointer to the unit.
- *	power	The power by which to raise "unit".  Must be greater than or 
+ *	power	The power by which to raise "unit".  Must be greater than or
  *		equal to -255 and less than or equal to 255.
  * Returns:
  *	NULL	Failure.  "ut_get_status()" will be:
@@ -933,7 +987,7 @@ ut_divide(
  *	else	Pointer to the resulting unit.  The pointer should be passed to
  *		ut_free() when the unit is no longer needed by the client.
  */
-ut_unit*
+EXTERNL ut_unit*
 ut_raise(
     const ut_unit* const	unit,
     const int			power);
@@ -944,7 +998,7 @@ ut_raise(
  *
  * Arguments:
  *	unit	Pointer to the unit.
- *	root	The root to take of "unit".  Must be greater than or 
+ *	root	The root to take of "unit".  Must be greater than or
  *		equal to 1 and less than or equal to 255.
  * Returns:
  *	NULL	Failure.  "ut_get_status()" will be:
@@ -956,7 +1010,7 @@ ut_raise(
  *	else	Pointer to the resulting unit.  The pointer should be passed to
  *		ut_free() when the unit is no longer needed by the client.
  */
-ut_unit*
+EXTERNL ut_unit*
 ut_root(
     const ut_unit* const	unit,
     const int			root);
@@ -1001,7 +1055,7 @@ ut_root(
  *			passed to ut_free() when the unit is no longer needed by
  *			the client.
  */
-ut_unit*
+EXTERNL ut_unit*
 ut_log(
     const double		base,
     const ut_unit* const	reference);
@@ -1034,11 +1088,11 @@ ut_log(
  *						"errno".
  *	else		Pointer to the unit corresponding to "string".
  */
-ut_unit*
+EXTERNL ut_unit*
 ut_parse(
     const ut_system* const	system,
     const char* const		string,
-    const ut_encoding		encoding);
+    ut_encoding		        encoding);
 
 
 /*
@@ -1051,7 +1105,7 @@ ut_parse(
  * Returns:
  *      "string", with all leading and trailing whitespace removed.
  */
-char*
+EXTERNL char*
 ut_trim(
     char* const	        string,
     const ut_encoding	encoding);
@@ -1093,7 +1147,7 @@ ut_trim(
  *			the number is equal to the size of the buffer, then the
  *			buffer is too small to have a terminating NUL character.
  */
-int
+EXTERNL int
 ut_format(
     const ut_unit* const	unit,
     char*		buf,
@@ -1113,7 +1167,7 @@ ut_format(
  *	UT_VISIT_ERROR	A error occurred in "visitor" while visiting "unit".
  *	UT_SUCCESS	Success.
  */
-ut_status
+EXTERNL ut_status
 ut_accept_visitor(
     const ut_unit* const		unit,
     const ut_visitor* const	visitor,
@@ -1135,7 +1189,7 @@ ut_accept_visitor(
  * Returns:
  *	The date encoded as a scalar value.
  */
-double
+EXTERNL double
 ut_encode_date(
     int		year,
     int		month,
@@ -1143,25 +1197,28 @@ ut_encode_date(
 
 
 /*
- * Encodes a time as a double-precision value.
+ * Encodes a time as a double-precision value. If an input value isn't within
+ * its allowed range, then zero is returned and `ut_get_status()` will return
+ * `UT_BAD_ARG`.
  *
- * Arguments:
- *	hours		The number of hours (0 = midnight).
- *	minutes		The number of minutes.
- *	seconds		The number of seconds.
- * Returns:
- *	The clock-time encoded as a scalar value.
+ * @param[in] hours    The number of hours (0 = midnight). `abs(hours)` must be
+ *                     less than 24.
+ * @param[in] minutes  The number of minutes. `abs(minutes)` must be less than
+ *                     60.
+ * @param[in] seconds  The number of seconds. `fabs(seconds)` must be less than
+ *                     or equal to 62.
+ * @return             The clock-time encoded as a scalar value.
  */
-double
+EXTERNL double
 ut_encode_clock(
-    int		hours,
-    int		minutes,
-    double	seconds);
+    int    hours,
+    int    minutes,
+    double seconds);
 
 
 /*
  * Encodes a time as a double-precision value.  The convenience function is
- * equivalent to "ut_encode_date(year,month,day) + 
+ * equivalent to "ut_encode_date(year,month,day) +
  * ut_encode_clock(hour,minute,second)"
  *
  * Arguments:
@@ -1174,7 +1231,7 @@ ut_encode_clock(
  * Returns:
  *	The time encoded as a scalar value.
  */
-double
+EXTERNL double
 ut_encode_time(
     const int		year,
     const int		month,
@@ -1198,7 +1255,7 @@ ut_encode_time(
  *      resolution      Pointer to the variable to be set to the resolution
  *                      of the decoded time in seconds.
  */
-void
+EXTERNL void
 ut_decode_time(
     double	value,
     int		*year,
@@ -1219,7 +1276,7 @@ ut_decode_time(
  * Returns the status of the last operation by the units module.  This function
  * will not change the status.
  */
-ut_status
+EXTERNL ut_status
 ut_get_status(void);
 
 
@@ -1230,7 +1287,7 @@ ut_get_status(void);
  * Arguments:
  *	status	The status of the units module.
  */
-void
+EXTERNL void
 ut_set_status(
     ut_status	status);
 
@@ -1246,7 +1303,7 @@ ut_set_status(
  *	else	The number of bytes of "fmt" and "arg" written excluding any
  *		terminating NUL.
  */
-int
+EXTERNL int
 ut_handle_error_message(
     const char* const	fmt,
     ...);
@@ -1258,12 +1315,12 @@ ut_handle_error_message(
  *
  * Arguments:
  *      handler		NULL or pointer to the error-message handler.  If NULL,
- *			then the handler is not changed.  The 
+ *			then the handler is not changed.  The
  *			currently-installed handler can be obtained this way.
  * Returns:
  *	Pointer to the previously-installed error-message handler.
  */
-ut_error_message_handler
+EXTERNL ut_error_message_handler
 ut_set_error_message_handler(
     ut_error_message_handler	handler);
 
@@ -1280,7 +1337,7 @@ ut_set_error_message_handler(
  *	else	The number of bytes of "fmt" and "arg" written excluding any
  *		terminating NUL.
  */
-int
+EXTERNL int
 ut_write_to_stderr(
     const char* const	fmt,
     va_list		args);
@@ -1295,7 +1352,7 @@ ut_write_to_stderr(
  * Returns:
  *	0	Always.
  */
-int
+EXTERNL int
 ut_ignore(
     const char* const	fmt,
     va_list		args);
